unit main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, EditBtn,
  Spin, ComCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    btnLoad: TButton;
    btnSave: TButton;
    edFileName: TFileNameEdit;
    edAppendix: TEdit;
    ImageList1: TImageList;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    ListView1: TListView;
    seKnownWidth: TSpinEdit;
    seKnownHeight: TSpinEdit;
    procedure btnLoadClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure seKnownWidthChange(Sender: TObject);
  private

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.btnLoadClick(Sender: TObject);
var
  img: TCustomBitmap;
  ext: String;
  i: Integer;
begin
  if (seKnownWidth.Value <= 0) then
    raise Exception.Create('Invalid known image width');
  if (seKnownHeight.Value <= 0) then
    raise Exception.Create('Invalid knon image height');

  ext := Lowercase(ExtractFileExt(edFileName.Text));
  if ext = '.png' then
    img := TPortableNetworkGraphic.Create
  else if ext = '.bmp' then
    img := TBitmap.Create
  else
    raise Exception.CreateFmt('%s images not supported.', [ext]);

  try
    img.LoadFromFile(edFileName.Text);
    if img.Width mod seKnownWidth.Value <> 0 then
      raise Exception.Create('Width of the loaded image is not a multiple of the known image width.');
    if img.Height mod seKnownHeight.Value <> 0 then
      raise Exception.Create('Height of the loaded image is not a multiple of the known image height.');
    ImageList1.Width := seKnownWidth.Value;
    ImageList1.Height := seKnownHeight.Value;
    ImageList1.Clear;
    ImageList1.AddSliced(img, img.Width div ImageList1.Width, img.Height div ImageList1.Height);

    ListView1.SmallImages := ImageList1;
    ListView1.Clear;
    for i := 0 to ImageList1.Count-1 do begin
      with ListView1.Items.Add do
      begin
        Caption := 'img' + IntToStr(i+1) + edAppendix.Text;
        ImageIndex := i;
      end;
    end;

    btnSave.Enabled := true;

  finally
    img.Free;
  end;
end;

procedure TForm1.btnSaveClick(Sender: TObject);
var
  img: TCustomBitmap;
  ext: String;
  outdir: String;
  i: Integer;
begin
  ext := Lowercase(ExtractFileExt(edFileName.Text));
  if ext = '.png' then
    img := TPortableNetworkGraphic.Create
  else if ext = '.bmp' then
    img := TBitmap.Create
  else
    raise Exception.CreateFmt('%s images not supported.', [ext]);

  outdir := ExtractFilePath(edFileName.Text) + 'output/';
  ForceDirectories(outdir);

  for i := 0 to ImageList1.Count-1 do
  begin
    ImageList1.GetBitmap(i, img);
    img.SaveToFile(outdir + ListView1.Items[i].Caption + ext);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  seKnownWidthChange(nil);
end;

procedure TForm1.seKnownWidthChange(Sender: TObject);
begin
  edAppendix.Text := Format('_%dx%d', [seKnownWidth.Value, seKnownHeight.Value]);
end;

end.

